
(function () {
  'use strict';

  var buttonWrap = document.createElement('div');
  var button = document.createElement('div');
  var buttonElement = document.createElement('button');
  var buttonSettings = document.createElement('button');
  var buttonClose = document.createElement('button');

  buttonWrap.className = 'snapaddy-global-crawl-button-wrap snapaddy-global-crawl-button-wrap--bottom-right';
  button.className = 'snapaddy-global-crawl-button';
  buttonElement.className = 'snapaddy-global-crawl-button__button';
  buttonSettings.className = 'snapaddy-global-crawl-button__settings';
  buttonClose.className = 'snapaddy-global-crawl-button__close';

  button.appendChild(buttonElement);
  button.appendChild(buttonSettings);
  button.appendChild(buttonClose);
  buttonWrap.appendChild(button);
  document.body.appendChild(buttonWrap);

  function enable() {
    buttonWrap.classList.add('snapaddy-global-crawl-button-wrap--visible');
    if (buttonWrap && buttonWrap.style) {
      buttonWrap.style.opacity = 1;
    }
  }

  function disable() {
    buttonWrap.classList.remove('snapaddy-global-crawl-button-wrap--visible');
    if (buttonWrap && buttonWrap.style) {
      buttonWrap.style.opacity = 0;
    }
  }

  chrome.storage.local.get('enableCrawlButton', function (data) {
    if (data.enableCrawlButton) {
      enable();
    } else {
      disable();
    }
  });

  chrome.runtime.onMessage.addListener((request) => {
    if (request.msg === 'enableCrawlButton') {
      if (request.enable) {
        enable();
      } else {
        disable();
      }
    }
  });

  const grabberButtonLookup = [
    {
      source: 'https://www.xing.com/profile/',
      buttonQuery: '#snapaddy-grabber-btn'
    },
    {
      source: 'https://www.linkedin.com/in/',
      buttonQuery: '#snapaddy-grabber-btn'
    }
  ];

  buttonElement.onclick = function () {
    if (snapADDYUtils.hasFABs()) {
      snapADDYUtils.toggleFABs();
      return;
    }

    for (var entry of grabberButtonLookup) {
      if (window.location.href.startsWith(entry.source)) {
        const grabberButton = document.querySelector(entry.buttonQuery);
        if (grabberButton) {
          grabberButton.dispatchEvent(new Event('click'));
        }
        return;
      }
    }

    chrome.runtime.sendMessage({
      crawl: true,
      data: {
        type: 'crawlBtn',
        link: window.location.toString()
      }
    });
  };

  buttonElement.onblur = function () {
    if (snapADDYUtils.hasFABs()) {
      setTimeout(() => {
        snapADDYUtils.hideFABs();
      }, 250);
    }
  };

  buttonSettings.onclick = function (e) {
    chrome.runtime.sendMessage({
      openPage: true,
      page: 'settings'
    });
    e.preventDefault();
    return false;
  };

  buttonClose.onclick = function () {
    if (chrome.i18n && confirm(chrome.i18n.getMessage('globalCrawlButton_removeAlert'))) {
      chrome.storage.local.set({ 'enableCrawlButton': false });
      chrome.runtime.sendMessage({ msg: 'enableCrawlButton', enable: false });
    }
    disable();
  };

  class snapADDYUtils {
    static addFAB(text, callback) {
      var $fabContainer = $('.snapaddy-global-crawl-button__fab-button-container');

      if (!$fabContainer.length) {
        var fabContainer = document.createElement('div');
        fabContainer.classList.add('snapaddy-global-crawl-button__fab-button-container');
        $('.snapaddy-global-crawl-button').append(fabContainer);

        $fabContainer = $(fabContainer);
      }

      var fab = document.createElement('button');
      fab.classList.add('snapaddy-global-crawl-button__fab-button');

      fab.innerHTML = text;
      fab.onclick = function () {

        if (callback) {
          try {
            callback();
          } catch (e) {
            console.error(e);
          }
        }

        snapADDYUtils.hideFABs();
        // $('.snapaddy-global-crawl-button__fab-button-container').removeClass('snapaddy-global-crawl-button--show-fab');
      };

      $fabContainer.append(fab);
    }

    static addLeadFAB(id, integration) {
      const text = chrome.i18n.getMessage('snapaddyUpdater_lead');
      this.addFAB(text, () => {
        const specific = 'lead';
        const properties = { 'lead_id': id };
        this.sendUpdaterRequest(integration, specific, properties);
      });
    }

    static addContactFAB(id, integration) {
      const text = chrome.i18n.getMessage('snapaddyUpdater_contact');
      this.addFAB(text, () => {
        const specific = 'contact';
        const properties = { 'contact_id': id };
        this.sendUpdaterRequest(integration, specific, properties);
      });

      const createContactText = chrome.i18n.getMessage('snapaddyUpdater_create_contact');
      this.addFAB(createContactText, () => {
        const specific = 'contact_create_contact';
        const properties = { 'contact_id': id };
        this.sendUpdaterRequest(integration, specific, properties);
      });
    }

    static addAccountFAB(id, integration) {
      const text = chrome.i18n.getMessage('snapaddyUpdater_account');
      this.addFAB(text, () => {
        const specific = 'account';
        const properties = { 'account_id': id };
        this.sendUpdaterRequest(integration, specific, properties);
      });



      const createContactText = chrome.i18n.getMessage('snapaddyUpdater_create_contact');
      this.addFAB(createContactText, () => {
        const specific = 'account_create_contact';
        const properties = { 'account_id': id };
        this.sendUpdaterRequest(integration, specific, properties);
      });
    }

    static removeFABs() {
      $('.snapaddy-global-crawl-button__fab-button-container').remove();
      $('.snapaddy-global-crawl-button__button').unbind('click', () => this.toggleFABs());
    }

    static hasFABs() {
      return $('.snapaddy-global-crawl-button__fab-button').length > 0;
    }


    static toggleFABs() {
      let crawlButton = document.getElementsByClassName('snapaddy-global-crawl-button__fab-button-container')[0];
      if (crawlButton && crawlButton.style) {
        let opacity = crawlButton.style.opacity;
        if (opacity !== '1') {
          crawlButton.style.opacity = 1;
          crawlButton.style['pointer-events'] = 'all';
        } else {
          crawlButton.style.opacity = 0;
          crawlButton.style['pointer-events'] = 'none';
        }
      }
    }

    static hideFABs() {
      let crawlButton = document.getElementsByClassName('snapaddy-global-crawl-button__fab-button-container')[0];
      if (crawlButton && crawlButton.style) {
        crawlButton.style.opacity = 0;
        crawlButton.style['pointer-events'] = 'none';
      }
    }

    static showFABs() {
      let crawlButton = document.getElementsByClassName('snapaddy-global-crawl-button__fab-button-container')[0];
      if (crawlButton && crawlButton.style) {
        crawlButton.style.opacity = 1;
        crawlButton.style['pointer-events'] = 'all';
      }
    }

    /**
     *
     * @param {string} integration
     * @param {string} specific
     * @param {{[key:string]:string}} properties
     */
    static sendUpdaterRequest(integration, specific, properties) {
      var data = {
        integration: integration,
        specific: specific,
        url: window.location.href
      };

      Object.keys(properties).forEach(function (key) {
        data[key] = properties[key];
      });
      console.info('sending: ', data);
      chrome.runtime.sendMessage({ msg: 'integration_update_to_grabber', data: data });
    }

    static onElementChange(selector, callback) {
      let innerHTML = null;
      new MutationObserver(() => {
        let elem = null;
        if (typeof selector === 'string') {
          elem = document.querySelector(selector);
        } else {
          elem = selector();
        }

        if (innerHTML && !elem) {
          callback(null);
          innerHTML = null;
        } else if (elem && elem.innerHTML !== innerHTML) {
          callback(elem);
          innerHTML = elem.innerHTML;
        }
      }).observe(document.body, {
        subtree: true,
        childList: true
      });
    }
  }

  class DynamicsCRMUpdater {

    constructor() {
      this.integration = 'dynamics';
    }

    getVisibleFrame() {
      let frames = window.document.querySelectorAll('iframe');
      let frame = null;

      for (let i = 0; i < frames.length; i++) {
        if (frames[i].style.visibility === 'visible') {
          frame = frames[i];
          break;
        }
      }
      return frame || null;
    }

    getRecordID() {
      let visibleFrame = this.getVisibleFrame();

      if (visibleFrame) {
        try {
          return visibleFrame.contentDocument.getElementById('crmFormSubmitId').value.slice(1, -1);
        } catch (e) {
          return null;
        }
      }
      return null;
    }

    addLeadFAB() {
      const text = chrome.i18n.getMessage('snapaddyUpdater_lead');
      snapADDYUtils.addFAB(text, () => {
        const specific = 'lead';
        const properties = { 'lead_id': this.getRecordID() };
        snapADDYUtils.sendUpdaterRequest(this.integration, specific, properties);
      });
    }

    addContactFAB() {
      const text = chrome.i18n.getMessage('snapaddyUpdater_contact');
      snapADDYUtils.addFAB(text, () => {
        const specific = 'contact';
        const properties = { 'contact_id': this.getRecordID() };
        snapADDYUtils.sendUpdaterRequest(this.integration, specific, properties);
      });

      const createContactText = chrome.i18n.getMessage('snapaddyUpdater_create_contact');
      snapADDYUtils.addFAB(createContactText, () => {
        const specific = 'contact_create_contact';
        const properties = { 'contact_id': this.getRecordID() };
        snapADDYUtils.sendUpdaterRequest(this.integration, specific, properties);
      });
    }

    addAccountFAB() {
      const text = chrome.i18n.getMessage('snapaddyUpdater_account');
      snapADDYUtils.addFAB(text, () => {
        const specific = 'account';
        const properties = { 'account_id': this.getRecordID() };
        snapADDYUtils.sendUpdaterRequest(this.integration, specific, properties);
      });

      const createContactText = chrome.i18n.getMessage('snapaddyUpdater_create_contact');
      snapADDYUtils.addFAB(createContactText, () => {
        const specific = 'account_create_contact';
        const properties = { 'account_id': this.getRecordID() };
        snapADDYUtils.sendUpdaterRequest(this.integration, specific, properties);
      });
    }


    init(disabledEntities) {
      snapADDYUtils.onElementChange(() => {
        let frame = this.getVisibleFrame();
        if (frame) {
          return frame.contentDocument.body;
        }
        return null;
      }, (element) => {
        snapADDYUtils.removeFABs();

        if (!element) {
          return;
        }

        let recordId = this.getRecordID();
        if (!disabledEntities.account && document.querySelector('#Tabnav_accts-main') && recordId) {
          this.addAccountFAB();
        } else if (!disabledEntities.contact && document.querySelector('#Tabnav_conts-main') && recordId) {
          this.addContactFAB();
        } else if (!disabledEntities.lead && document.querySelector('#Tabnav_leads-main') && recordId) {
          this.addLeadFAB();
        }
        if (snapADDYUtils.hasFABs()) {
          enable();
        } else {
          disable();
        }

      });
    }
  }

  class SapC4cCrmUpdater {
    getParameterByName(name) {
      var match = new RegExp('[?&]' + name + '=([^&]*)').exec(window.location.search);
      return match && decodeURIComponent(match[1].replace(/\+/g, ' '));
    }

    constructor() {
      this.integration = 'sap_c4c';
      // set up an observer for the title element
      // const target = document.querySelector('head > title');
      // const observer = new window.WebKitMutationObserver((mutations) => {
      //   mutations.forEach((mutation) => {
      //     console.log('new title:', mutation.target.textContent);
      //   });
      // });
      // observer.observe(target, { subtree: true, characterData: true, childList: true });
      /* IDEA: Newclient (HTML5) fetch changes by observing the title */

    }

    decodeURL() {
      const elements = window.document.URL.split('/');
      try {
        let encodedObject;
        for (let i = elements.length - 1; i > -1; i--) {
          if (elements[i].length === 1) { break; } else {
            if (!encodedObject) {
              encodedObject = elements[i];
            } else {
              encodedObject = elements[i].concat('/' + encodedObject);
            }
          }
        }
        const decodedObject = atob(encodedObject);
        return JSON.parse(decodedObject);
      } catch (e) {
        console.error(e);
        return null;
      }
    }

    init() {
      window.onpopstate = () => {
        snapADDYUtils.removeFABs();
        disable();
        try {
          let recordId = null;

          const object = this.decodeURL();
          if (object) {

            const getFromString = function (resp) {
              try {

                if (resp) {
                  if (!resp.includes('?xml version')) {
                    if (object.thingparams || object.inportparam) {
                      return object.thingparams ? object.thingparams.Key : object.inportparam.Key;
                    }
                  } else {

                    // sometimes the id is contained in an xml structure in the following format:
                    //<Data><${idProperty}>${RecordId}</${idProperty}></Data>
                    const regex = /<Data><.+?>(.*?)<.+?><\/Data>/igm;
                    const matches = regex.exec(resp);
                    if (!!matches && !!matches[1]) {
                      return matches[1];
                    }

                    return resp.split('[')[1].split(']')[0];
                  }
                }

              } catch (e) {
                console.warn('Error while reading RecordID: ', e);
                return null;
              }
            }

            if (object.thingparams) {
              recordId = getFromString(object.thingparams.Key || '');

            } else if (object.inportparam) {
              if (object.inportparam.length) {
                recordId = object.inportparam
                  .filter(function (param) { return param.startsWith('Key'); })
                  .map(function (param) { return param.split('=')[1] })
                  .filter(Boolean)
                  .pop();

              } else {
                recordId = getFromString(object.inportparam.Key || '');

              }
            }

          } else {
            return; //Not viewing Account/Contact or Lead
          }
          if (recordId && object.target) {
            if (object.target.includes('COD_Account')) {
              enable();
              snapADDYUtils.addAccountFAB(recordId, 'sap_c4c');

            } else if (object.target.includes('COD_Contact')) {
              enable();
              snapADDYUtils.addContactFAB(recordId, 'sap_c4c');

            } else if (object.target.includes('COD_Mkt_Prospect')) {
              enable();
              snapADDYUtils.addLeadFAB(recordId, 'sap_c4c');

            }
          }

        } catch (error) {
          console.error(error);
        }
      }
    }
  }



  // the Grabber saves the updaterConfig
  // we load it here and inject the needed file if the url matches
  chrome.storage.local.get('SNAPADDY_UPDATER_CONFIG', function (items) {
    let configArray = items.SNAPADDY_UPDATER_CONFIG;
    if (configArray && configArray.length > 0) {

      configArray.forEach(config => {
        if (window.location.href.indexOf(config.url) > -1 && config.showUpdaterButton) {
          enable();
          $('.snapaddy-global-crawl-button__button').addClass('snapaddy-global-crawl-button__button--update');

          if (config.key === 'dynamics') {
            new DynamicsCRMUpdater().init(config.disabledEntities);
          } else if (config.key === 'sap_c4c') {
            new SapC4cCrmUpdater().init();
          }
        }
      });
    }
  });


})();
