'use strict';
window.addEventListener('load', function () {
  createUI();
}, false);

let emails = [];
let emailList = document.createElement('ul');
let output = document.createElement('div');
const REGEX = /^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;

const MAX_LINKS = 20;
let linksToParse = MAX_LINKS;

function createUI() {
  let outerDiv = document.createElement('div');
  let time = '750ms';

  emailList.style.listStyle = 'none';
  outerDiv.appendChild(emailList);
  outerDiv.style.position = 'fixed';
  outerDiv.style.right = '0px';
  outerDiv.style.width = '100px';
  outerDiv.style.bottom = '0px';
  outerDiv.style.height = '60px';
  outerDiv.style.padding = '0 10px';
  outerDiv.style.textAlign = 'left';
  outerDiv.style.backgroundColor = 'transparent';
  outerDiv.style.color = '#FFF';
  outerDiv.style.zIndex = '10000';
  outerDiv.style.overflowX = 'auto';
  outerDiv.style.transition = 'width ' + time + ', height ' + time + ', background-color ' + time;


  output.style.position = 'absolute';
  output.style.bottom = '5px';
  output.style.left = '5%';
  output.style.width = '90%';
  output.style.height = '50px';
  output.innerHTML = '';
  outerDiv.appendChild(output);

  let btn = document.createElement('button');
  btn.onclick = function () {
    outerDiv.style.width = '300px';
    outerDiv.style.height = '100%';
    outerDiv.style.backgroundColor = '#858d00';
    readPage();
    outerDiv.removeChild(btn);
  };
  btn.style.position = 'absolute';
  btn.style.bottom = '5px';
  btn.style.left = '5%';
  btn.style.width = '90%';
  btn.style.height = '50px';
  btn.innerHTML = 'parse';
  btn.className = 'btn btn-primary-inverse btn-lg btn-block';
  outerDiv.appendChild(btn);

  document.body.appendChild(outerDiv);
}

function readPage() {
  // read all mail addresses from this page
  parseHTML(document.body.innerHTML);

  // get all links tags
  let siteLinks = [];
  let linkNodeList = document.querySelectorAll('a');

  // filter for internal links on same domain
  for (let i = 0; i < linkNodeList.length; ++i) {
    let item = linkNodeList[i];
    if (item.href.indexOf(location.host) > -1) {
      siteLinks.push(item.href);
    }
  }

  // remove duplicates
  siteLinks = eliminateDuplicates(siteLinks);

  // just parse the first n links
  if (siteLinks.length < MAX_LINKS || MAX_LINKS === -1) {
    linksToParse = siteLinks.length;
  }


  let cnt = 1;
  siteLinks.forEach(link => {
    let i = cnt++;
    let time = 1000 * i;

    // if its a mailto link
    if (link.toLowerCase().indexOf('mailto') > -1) {
      handleMail(link.substr(link.indexOf(':') + 1));
      return;
    }

    if (i <= linksToParse) {
      setTimeout(function () {
        checkPage(link);
        output.innerHTML = ('snapADDY - parsing page nr: ' + i);
        if (i === linksToParse) {
          output.innerHTML = ('Found <b>' + emails.length + '</b> mail addresses');
        }
      }, time);
    }
  });



}

function checkPage(url) {
  $.ajax(url).done(function (data) {
    parseHTML(data);
  });
}

function eliminateDuplicates(arr) {
  let i,
    len = arr.length,
    out = [],
    obj = {};

  for (i = 0; i < len; i++) {
    obj[arr[i]] = 0;
  }
  for (i in obj) {
    out.push(i);
  }
  return out;
}

function parseHTML(html) {
  let bodyText = html;
  let emailRegex = /(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))/;
  let match = null;

  bodyText = bodyText.replace(/>/g, '> ');
  bodyText = bodyText.replace(/</g, '< ');
  bodyText = bodyText.replace(/;/g, '; ');
  bodyText = bodyText.replace(/}/g, '} ');
  bodyText = bodyText.replace(/{/g, '{ ');

  while (match = emailRegex.exec(bodyText)) {
    //-- store in array the found match email
    handleMail(match[0]);

    //-- remove the found email and continue search if there are still emails
    bodyText = bodyText.replace(match[0], "");
  }
}

function handleMail(mail) {
  if (emails.indexOf(mail) < 0) {
    let li = document.createElement('li');
    li.innerHTML = mail;
    emailList.appendChild(li);

    emails.push(mail);
  }
}
