var PAGETYPE_NONE = 0;
var PAGETYPE_XING = 1;
var PAGETYPE_LINKEDIN = 2;

var LINKEDIN_NO_PROFILE_PICTURE = "https://static.licdn.com/scds/common/u/images/themes/katy/ghosts/person/ghost_person_60x60_v1.png";
var XING_NO_PROFILE_PICTURE = "https://x2.xingassets.com/assets/frontend_minified/img/users/nobody_m.256x256.jpg";

function ProfileReader(profileLink){
	this.profileLink = profileLink;
	this.name = "";
	this.company = "";
	this.companyProfileLink = "";
	this.position = "";
	this.profilePicLink = "";
	this.html = ""
	this.isLoggedIn = false;
	
	if(profileLink.indexOf("https://www.xing.com/profile/") > -1){
		this.pageType = PAGETYPE_XING;
	} else if(profileLink.indexOf(".linkedin.com") > -1 && profileLink.indexOf("/dir/") == -1 && profileLink.indexOf("/company/") == -1){
		this.pageType = PAGETYPE_LINKEDIN;
	} else {
		this.pageType = PAGETYPE_NONE;
	}
};

ProfileReader.prototype.getHTML = function(url, callback) {
	var xmlHttp = null;
	
	try {
		xmlHttp = new XMLHttpRequest();
	} catch(e) {
		// Houston, we got a problem
		console.info("Error: XMLHttpRequest not supported");
	}
	
	if (xmlHttp) {
		xmlHttp.open('GET', url, true);
		xmlHttp.setRequestHeader("Content-Type", "text/html");
		xmlHttp.setRequestHeader("Cache-Control", "max-age=0");
		xmlHttp.onreadystatechange = function () {
			if (xmlHttp.readyState == 4) {
				callback(xmlHttp.responseText);
			}
		};
		xmlHttp.send(null);
	}
}

ProfileReader.prototype.readName = function() {
	if(this.pageType == PAGETYPE_XING){
		var cutOff = "<h1 itemprop=\"name\" class=\"fl mr10\">";
		var startString = this.html.substring(this.html.indexOf(cutOff) + cutOff.length);
		return startString.substring(0,startString.indexOf("<"));
	} else if(this.pageType == PAGETYPE_LINKEDIN){
		var cutOff = "<span class=\"full-name\" dir=\"auto\">";
		var startString = this.html.substring(this.html.indexOf(cutOff) + cutOff.length);
		return startString.substring(0,startString.indexOf("<"));
	} else{
		return "";
	}
}

ProfileReader.prototype.readPosition = function() {
	if(this.pageType == PAGETYPE_XING){
		var cutOff = "<strong itemprop=\"jobTitle\" class=\"job-title\">";
		var startString = this.html.substring(this.html.indexOf(cutOff) + cutOff.length);
		return startString.substring(0,startString.indexOf("<"));
	} else if(this.pageType == PAGETYPE_LINKEDIN){
		if(this.html.indexOf("background-experience-container") == -1)
			return "";
		
		var cutOffs = ["<header>","<h4","<a",">"];
		var cutString = this.html;
		for(var i = 0; i<cutOffs.length; i++) 
			cutString = cutString.substring(cutString.indexOf(cutOffs[i]) + cutOffs[i].length)

		return cutString.substring(0,cutString.indexOf("<"));
	} else{
		return "";
	}
}

ProfileReader.prototype.readCompanyProfileLink = function() {
	if(this.pageType == PAGETYPE_XING) {
		var cutOff = "<a class=\"company-name inv-link\"";
		var startString = this.html.substring(this.html.indexOf(cutOff) + cutOff.length);
		startString = startString.substring(startString.indexOf("href=") + "href=".length);
		startString = startString.substring(1,startString.indexOf(">")-1);
		
		if(startString.indexOf("xing.com") == -1){
			if(startString.indexOf("/") == 0){
				startString = startString.substring(1);
			}
			startString = "http://www.xing.com/" + startString;
		}
		return startString;		
	} else if(this.pageType == PAGETYPE_LINKEDIN){	
		if(this.html.indexOf("background-experience-container") == -1)
			return "";	
		
		var cutString = this.html;
		var cutOffs = ["overview-summary-current", "<a", "\""];
		for(var i = 0; i<cutOffs.length; i++) {
			cutString = cutString.substring(cutString.indexOf(cutOffs[i]) + cutOffs[i].length)
			console.info("123" + this.name + " - " + cutOffs[i] + " - " + cutString)
		}
		cutString = cutString.substring(0,cutString.indexOf("\""));
		
		if(cutString.indexOf("?") > -1){
			if(cutString.indexOf("vsearch") < 0){
				cutString = cutString.substring(0, cutString.indexOf("?"));
			} else {
				cutString = cutString.replace(/&amp;/g, '&');
				cutString = cutString.substring(0, cutString.indexOf("&trk"));
			}
		}
		
		if(cutString.indexOf("linkedin.com") == -1){
			if(cutString.indexOf("/") == 0){
				cutString = cutString.substring(1);
			}
			cutString = "http://www.linkedin.com/" + cutString;
		}
		
		if(cutString.indexOf("www.linkedin.com/company/") == -1){
			console.info(this.name + ": No official LinkedIn-Companypage found for company: " + this.company + "!");
			return "";
		}
		
		return cutString;
	} else{
		return "";
	}
}


ProfileReader.prototype.readCompany = function() {
	if(this.pageType == PAGETYPE_XING){
		var cutOff = "<a class=\"company-name inv-link\"";
		var startString = this.html.substring(this.html.indexOf(cutOff) + cutOff.length);
		startString = startString.substring(startString.indexOf(">") + ">".length);
		return startString.substring(0,startString.indexOf("<"));
	} else if(this.pageType == PAGETYPE_LINKEDIN){
		if(this.html.indexOf("background-experience-container") == -1)
			return "";	
		
		var cutOff = "<header";
		var startString = this.html.substring(this.html.indexOf(cutOff) + cutOff.length);
			
		cutOff = "<h5>";
		startString = startString.substring(startString.indexOf(cutOff) + cutOff.length);
		
		cutOff = ">";
		startString = startString.substring(startString.indexOf(cutOff) + cutOff.length);

		if(startString.indexOf("<strong") == 0){
			cutOff = "<a";
			startString = startString.substring(startString.indexOf(cutOff) + cutOff.length);		
			cutOff = ">";
			startString = startString.substring(startString.indexOf(cutOff) + cutOff.length);
		}
		return startString.substring(0,startString.indexOf("<"));
	} else{
		return "";
	}
}

ProfileReader.prototype.readProfilePic = function(){
	if(this.pageType == PAGETYPE_XING){
		var cutOff = "<img class=\"user-image\"";
		var src = "src=\"";
		
		if(this.html.indexOf(cutOff) < 0){
			return XING_NO_PROFILE_PICTURE;
		}
		
		var startString = this.html.substring(this.html.indexOf(cutOff) + cutOff.length);
		startString = startString.substring(startString.indexOf(src) + src.length);
		return startString.substring(0,startString.indexOf("\""));
	} else if(this.pageType == PAGETYPE_LINKEDIN){
		var cutOff = "<div class=\"profile-picture\">";
		var src = "src='";
		var end = "'";
		
		if(this.html.indexOf(cutOff) < 0){
			return LINKEDIN_NO_PROFILE_PICTURE;
		}
		var startString = this.html.substring(this.html.indexOf(cutOff) + cutOff.length);
		
		if(startString.indexOf(src) < 0) {
			src = "src=\"";
			end = "\"";
		}
		
		startString = startString.substring(startString.indexOf(src) + src.length);
		
		return startString.substring(0,startString.indexOf(end));
	} else{
		return "";
	}
}

ProfileReader.prototype.read = function(callback) {
	var ref = this;
	
	if(this.pageType == PAGETYPE_NONE)
		return;
	console.info(this.profileLink);
	this.getHTML(this.profileLink, function(html){
    
		if(ref.pageType == PAGETYPE_LINKEDIN){
			ref.isLoggedIn = html.indexOf("linkedin.com/uas/login-submit") < 0;
		}else if(ref.pageType == PAGETYPE_XING){
			ref.isLoggedIn = html.indexOf("login.xing.com/login") < 0;
		}
	
		if(!ref.isLoggedIn){
			return;
		}
		
		ref.html = html;
				
		ref.name = ref.readName();
		ref.position = ref.readPosition();
		ref.company = ref.readCompany();
		ref.companyProfileLink = ref.readCompanyProfileLink();
		ref.profilePicLink = ref.readProfilePic();
			
		callback(ref);
	});
};

ProfileReader.prototype.getIsLoggedIn = function() {
	return this.isLoggedIn;
}

ProfileReader.prototype.getPageType = function() {
  return this.pageType;
};

ProfileReader.prototype.getName = function() {
  return this.name;
};

ProfileReader.prototype.getCompany = function() {
  return this.company;
};

ProfileReader.prototype.getCompanyProfileLink = function() {
  return this.companyProfileLink;
};

ProfileReader.prototype.getPosition = function() {
  return this.position;
};

ProfileReader.prototype.getProfilePic = function() {
  return this.profilePicLink;
};

ProfileReader.prototype.getProfileLink = function() {
  return this.profileLink;
};