(function () {
  'use strict';

  var getLeadIdFromURL = function () {
    var matches = /.*?app\.close\.(?:io|com)\/lead\/(.*)/.exec(window.location.href);
    if (matches && matches.length > 1) {
      return matches[1].replace(/\//, '');
    }
    return null;
  };

  var createLeadButtons = function () {
    var leadId = getLeadIdFromURL();
    if (!leadId) { return false; }

    if (document.querySelector('.snapaddy-btn-group') || document.querySelector('snapaddy-button')) {
      return true;
    }

    // CREATE LEAD ACTION BUTTON
    function createLeadAction() {
      try {

        var leadActions = document.querySelector('.lead-actions');

        var btnGroup = document.createElement('div');
        btnGroup.className = 'btn-group snapaddy-btn-group';


        btnGroup.innerHTML = `
          <button class="btn dropdown-toggle snapaddy-button" data-toggle="dropdown" rel="tooltip" title="snapaddy Updater" data-original-title="snapaddy Updater">
            <span class="icon"></span> snapADDY Updater <span class="caret"></span>
          </button>
          <ul class="dropdown-menu">
            <li>
              <a id="update_lead" href="#" title="Lead updaten">Lead updaten</a>
              <a id="lead_create_contact" href="#" title="neuen Kontakt anlegen">neuen Kontakt anlegen</a>
            </li>
          </ul>
        `;

        leadActions.insertBefore(btnGroup, leadActions.firstChild);
        document.getElementById('update_lead').onclick = function () {
          chrome.runtime.sendMessage({
            msg: 'integration_update_to_grabber', data: {
              integration: 'closeio',
              specific: 'lead',
              lead_id: leadId
            }
          });
        };
        document.getElementById('lead_create_contact').onclick = function () {
          chrome.runtime.sendMessage({
            msg: 'integration_update_to_grabber', data: {
              integration: 'closeio',
              specific: 'lead_create_contact',
              lead_id: leadId
            }
          });
        };
      } catch (e) { return false; }
      return true;
    }

    // CREATE CONTACT ACTION BUTTON
    function createContactAction() {
      try {
        var contactsListEntries = document.querySelectorAll('.contacts-list li');

        var createButton = function (listEntry) {
          var itemId = listEntry.dataset.itemId;

          var boxButtonWrapper = document.querySelector('.contacts-list li[data-item-id="' + itemId + '"] .box-buttons');

          var snapaddyBoxButton = document.createElement('a');
          snapaddyBoxButton.className = 'snapaddy-box-button';
          snapaddyBoxButton.setAttribute('rel', 'tooltip');
          snapaddyBoxButton.dataset.originalTitle = 'Kontakt updaten';
          snapaddyBoxButton.title = 'Kontakt updaten';

          snapaddyBoxButton.onclick = function () {
            chrome.runtime.sendMessage({
              msg: 'integration_update_to_grabber', data: {
                integration: 'closeio',
                specific: 'contact',
                lead_id: leadId,
                contact_id: itemId
              }
            });
          };
          boxButtonWrapper.appendChild(snapaddyBoxButton);
        };

        for (var i = 0; i < contactsListEntries.length; i++) {
          createButton(contactsListEntries[i]);
        }
      } catch (e) { return false; }
      return true;
    }

    if (!createLeadAction()) {
      return false;
    }
    if (!createContactAction()) {
      return false;
    }

    return true;
  };

  var timeoutId = null;
  var update = function () {
    if (timeoutId) {
      clearTimeout(timeoutId);
    }

    chrome.storage.local.get('SNAPADDY_UPDATER_CONFIG', function (items) {
      let configArray = items.SNAPADDY_UPDATER_CONFIG;
      if (configArray && configArray.length > 0) {
        configArray.forEach((config) => {
          if (config.key === 'closeio') {
            if (!config.showUpdaterButton || !createLeadButtons()) {
              timeoutId = setTimeout(update, 100);
            } else {
              document.addEventListener('DOMSubtreeModified', update, { capture: false, once: true });
            }
          }
        });
      }
    });
  };

  setTimeout(update, 100);

})();
