(function () {
  'use strict';

  var updateContactLabel = chrome.i18n.getMessage('snapaddyUpdater_contact');
  var updateOrganizationLabel = chrome.i18n.getMessage('snapaddyUpdater_organization');
  var updateDealLabel = chrome.i18n.getMessage('snapaddyUpdater_deal');
  var createContactLabel = chrome.i18n.getMessage('snapaddyUpdater_create_contact');

  var newContactDescription = chrome.i18n.getMessage('snapaddyUpdater_create_contact_description');
  var updateDealDescription = chrome.i18n.getMessage('snapaddyUpdater_deal_description');
  var updateContactDescription = chrome.i18n.getMessage('snapaddyUpdater_contact_description');
  var updateOrganizationDescription = chrome.i18n.getMessage('snapaddyUpdater_organization_description');

  var mutationObserver;

  function createUpdateButton() {

    var contactId = null;
    var orgaId = null;
    var dealId = null;

    var personMatches = /pipedrive.com\/person\/(.+)/.exec(window.location.href.replace(/#/g, ''));
    if (personMatches && personMatches.length) {
      contactId = personMatches[1];
    }

    var orgaMatches = /pipedrive.com\/organization\/(.+)/.exec(window.location.href.replace(/#/g, ''));
    if (orgaMatches && orgaMatches.length) {
      orgaId = orgaMatches[1];
    }

    var dealMatches = /pipedrive.com\/deal\/(.+)/.exec(window.location.href.replace(/#/g, ''));
    if (dealMatches && dealMatches.length) {
      dealId = dealMatches[1];
    }

    chrome.storage.local.get('SNAPADDY_UPDATER_CONFIG', function (items) {
      let configArray = items.SNAPADDY_UPDATER_CONFIG;
      if (configArray && configArray.length > 0) {
        configArray.forEach((config) => {
          if (config.key === 'pipedrive') {
            if (config.showUpdaterButton) {
              if (contactId) {
                addContactButton(contactId);
              }
              if (orgaId) {
                addOrganizationButton(orgaId);
              }
              if (dealId) {
                addDealButton(dealId);
              }
            }
          }
        });
      }
    });
  }

  function getOrganizationIdFromDOM() {
    var href = $('.viewContainer:not([style*="none"]) .organizationFields .itemTitle h3 a').prop('href');
    var REGEX = /(?:.*\/)(.*?)$/;
    var matches = REGEX.exec(href);
    return (matches && matches.length > 1) ? matches[1] : null;
  }

  function getPersonIdFromDOM() {
    var href = $('.viewContainer:not([style*="none"]) .personFields .itemTitle h3 a').prop('href');
    var REGEX = /(?:.*\/)(.*?)$/;
    var matches = REGEX.exec(href);
    return (matches && matches.length > 1) ? matches[1] : null;
  }

  function fillPopOver(options) {
    $('.viewContainer:not([style*="none"]) #snapaddy-dropdown-menu .visibilityOptions').empty();
    for (var i = 0; i < options.length; i++) {
      $('.viewContainer:not([style*="none"]) #snapaddy-dropdown-menu .visibilityOptions').append('<li id="' + options[i].id + '"><h5>' + options[i].title + '</h5><p>' + options[i].description + '</p></li>');
      $('#' + options[i].id).click(options[i].callback);
    }
  }

  function showPopover() {
    var dropdownMenu = $('.viewContainer:not([style*="none"]) #snapaddy-dropdown-menu');
    dropdownMenu.toggleClass('visible');
    setTimeout(function () {
      if (dropdownMenu.hasClass('visible')) {
        window.onclick = function () {
          if (dropdownMenu.hasClass('visible')) {
            dropdownMenu.removeClass('visible');
            window.onclick = function () { };
          }
        };
      }
    }, 10);
  }

  function addContactButton(id) {
    if ($('.actions').length === 0) {
      return;
    }

    setTimeout(function () {
      destroyMutationObserver();

      $('.snapaddy-button').each(function () {
        $(this).remove();
      });

      $('#snapaddy-dropdown-menu').each(function () {
        $(this).remove();
      });


      // add the snapADDY Updater button
      $('.personDetails .actions').each(function (idx, value) {
        var updaterButton = $(value).find('.buttonAddDeal');
        updaterButton.after('<span class="input buttonAddDeal snapaddy-button" id="snapaddy_updater"><button class="button"><span class="icon">snapADDY Updater</span></button></span>');
      });

      $('.snapaddy-button').each(function () {
        $(this).append('<div id="snapaddy-dropdown-menu"><ul class="visibilityOptions"></ul></div>');
        $(this).click(function (e) {
          e.stopPropagation();
          destroyMutationObserver();
          showPopover();
          createMutationObserver();
        });
      });

      var org_id = getOrganizationIdFromDOM();
      var options = [];

      options.push({
        id: 'contact_01',
        title: updateContactLabel,
        description: updateContactDescription,
        callback: function () {
          chrome.runtime.sendMessage({
            msg: 'integration_update_to_grabber', data: {
              integration: 'pipedrive',
              specific: 'person',
              domain: window.location.href,
              person_id: id,
              org_id: getOrganizationIdFromDOM()
            }
          });
        }
      });

      if (org_id) {
        options.push({
          id: 'contact_02',
          title: createContactLabel,
          description: newContactDescription,
          callback: function () {
            chrome.runtime.sendMessage({
              msg: 'integration_update_to_grabber', data: {
                integration: 'pipedrive',
                specific: 'person_create_contact',
                domain: window.location.href,
                person_id: id,
                org_id: getOrganizationIdFromDOM()
              }
            });
          }
        });
      }

      fillPopOver(options);
      createMutationObserver();
    }, 100);
  }

  function addOrganizationButton(id) {
    if ($('.actions').length === 0) {
      return;
    }

    setTimeout(function () {
      destroyMutationObserver();

      $('.snapaddy-button').each(function () {
        $(this).remove();
      });

      $('#snapaddy-dropdown-menu').each(function () {
        $(this).remove();
      });


      // add the snapADDY Updater button
      $('.organizationDetails .actions').each(function (idx, value) {
        var updaterButton = $(value).find('.buttonAddDeal');
        updaterButton.after('<span class="input buttonAddDeal snapaddy-button" id="snapaddy_updater"><button class="button"><span class="icon">snapADDY Updater</span></button></span>');
      });

      $('.snapaddy-button').each(function () {
        $(this).append('<div id="snapaddy-dropdown-menu"><ul class="visibilityOptions"></ul></div>');
        $(this).click(function (e) {
          e.stopPropagation();
          destroyMutationObserver();
          showPopover();
          createMutationObserver();
        });
      });

      var org_id = getOrganizationIdFromDOM();
      var options = [];

      options.push({
        id: 'orga_01',
        title: updateOrganizationLabel,
        description: updateOrganizationDescription,
        callback: function () {
          chrome.runtime.sendMessage({
            msg: 'integration_update_to_grabber', data: {
              integration: 'pipedrive',
              specific: 'organization',
              domain: window.location.href,
              org_id: id
            }
          });
        }
      });
      options.push({
        id: 'orga_02',
        title: createContactLabel,
        description: newContactDescription,
        callback: function () {
          chrome.runtime.sendMessage({
            msg: 'integration_update_to_grabber', data: {
              integration: 'pipedrive',
              specific: 'organization_create_contact',
              domain: window.location.href,
              org_id: id
            }
          });
        }
      });

      fillPopOver(options);

      createMutationObserver();
    }, 100);
  }

  function addDealButton(id) {
    if ($('.actions').length === 0) {
      return;
    }

    setTimeout(function () {
      destroyMutationObserver();

      $('.snapaddy-button').each(function () {
        $(this).remove();
      });

      $('#snapaddy-dropdown-menu').each(function () {
        $(this).remove();
      });

      let personId = getPersonIdFromDOM();
      let orgaId = getOrganizationIdFromDOM();

      var options = [];

      if (personId && orgaId) {
        options.push({
          id: 'deal_02',
          title: updateDealLabel,
          description: updateDealDescription,
          callback: function () {
            chrome.runtime.sendMessage({
              msg: 'integration_update_to_grabber', data: {
                integration: 'pipedrive',
                specific: 'deal',
                domain: window.location.href,
                deal_id: id,
                org_id: getOrganizationIdFromDOM(),
                person_id: getPersonIdFromDOM()
              }
            });
          }
        });
      }

      if (orgaId) {
        options.push({
          id: 'deal_01',
          title: createContactLabel,
          description: newContactDescription,
          callback: function () {
            chrome.runtime.sendMessage({
              msg: 'integration_update_to_grabber', data: {
                integration: 'pipedrive',
                specific: 'deal_create_contact',
                domain: window.location.href,
                deal_id: id,
                org_id: getOrganizationIdFromDOM(),
                person_id: getPersonIdFromDOM()
              }
            });
          }
        });
      }

      if (options.length > 0) {
        // add the snapADDY Updater button
        $('.dealDetails .actions').each(function (idx, value) {
          var updaterButton = $(value).find('.stateActions');
          updaterButton.after('<span class="input buttonAddDeal snapaddy-button" id="snapaddy_updater"><button class="button"><span class="icon">snapADDY Updater</span></button></span>');
        });

        $('.snapaddy-button').each(function () {
          $(this).append('<div id="snapaddy-dropdown-menu"><ul class="visibilityOptions"></ul></div>');
          $(this).click(function (e) {
            e.stopPropagation();
            destroyMutationObserver();
            showPopover();
            createMutationObserver();
          });
        });

        fillPopOver(options);
      }
      createMutationObserver();
    }, 100);
  }

  function createMutationObserver() {
    mutationObserver = new MutationObserver(createUpdateButton);
    mutationObserver.observe(document.body, {
      subtree: true,
      childList: true
    });
  }

  function destroyMutationObserver() {
    mutationObserver.disconnect();
  }

  window.addEventListener('load', function () {
    createMutationObserver();
  }, false);

})();
