(function () {
  'use strict';
  console.warn('SALESFORCE_LIGHTNING SCRIPT LAODED');
  const updateContactLabel = chrome.i18n.getMessage('snapaddyUpdater_contact');
  const createContactLabel = chrome.i18n.getMessage('snapaddyUpdater_create_contact');
  const updateLeadLabel = chrome.i18n.getMessage('snapaddyUpdater_lead');
  const updateAccountLabel = chrome.i18n.getMessage('snapaddyUpdater_account');

  var mutationObserver;

  var getObjectIdFromURL = function () {
    var matches = new RegExp('.*?\/sObject\/(.*?)\/.*').exec(window.location);
    if (!matches) {
      matches = new RegExp('lightning\/r\/.*\/(.*)\/.*').exec(window.location);
    }

    return matches.length > 1 ? matches[1] : null;
  }

  const isActive = (entity) => {
    try {
      const href = document.querySelectorAll('.slds-context-bar__item.slds-shrink-none.slds-is-active a')[0].href;
      const active = href && href.toLowerCase().indexOf(entity) > -1 && !!getObjectIdFromURL();
      return active;
    } catch (e) {
      return false;
    }
  };

  const isRecordPage = () => {
    return !!document.querySelector('.slds-page-header_record-home');
  };

  function fillMenuList(options) {
    var addOption = function (option) {
      var liHtml = `
        <li class="uiMenuItem" id="${option.id}">
          <a tabindex="0" title="${option.title}">
            <div title="${option.title}" class="forceActionLink">${option.title}</div>
          </a>
        </li>
      `;

      $('.snapaddy-menu ul').append(liHtml);
      $('.snapaddy-menu ul #' + option.id).click(option.callback);
    };

    var html = `
      <div class="snapaddy-menu actionMenu popupTargetContainer uiPopupTarget uiMenuList forceActionsDropDownMenuList uiMenuList--left uiMenuList--default">
        <div class="actionMenu">
          <ul class="scrollable"></ul>
        </div>
      </div>
    `;
    $('.snapaddy-button').append(html);
    for (var i in options) {
      addOption(options[i]);
    }
  }

  function createMenu(options) {

    var text = 'snapADDY Updater';
    var buttonTitle = '';

    var html = `
      <li class="snapaddy-button slds-button slds-button--neutral">
        <a href="javascript:void(0);" title="${text}" class="forceActionLink">
          <div title="${text}" class="icon">
            ${text}
            <span title="${buttonTitle}" class="forceIconDeprecated forceIcon" data-key="down" data-icon="" alt="${buttonTitle}"><span class="assistiveText">${buttonTitle}</span></span>
          </div>
        </a>
      </li>
    `;

    $('.snapaddy-button').remove();

    const buttonBar = $('.oneContent.active .slds-page-header .slds-button-group.oneActionsRibbon.forceActionsContainer');

    const amIOnARecordPage = isRecordPage();

    if (!buttonBar || !amIOnARecordPage) {
      return false;
    }

    buttonBar.prepend(html);

    fillMenuList(options);

    $('.snapaddy-button').click(function (e) {
      e.stopPropagation();
      $('.snapaddy-menu').toggleClass('visible');
    });

    $(document).click(function () {
      $('.snapaddy-menu').removeClass('visible');
    });

    return true;
  }

  function createLeadButtons() {
    if ($('.oneContent.active .slds-page-header .slds-button-group.oneActionsRibbon.forceActionsContainer').length === 0) {
      return;
    }

    setTimeout(function () {
      destroyMutationObserver();

      $('.snapaddy-button').each(function () {
        $(this).remove();
      });

      $('#snapaddy-dropdown-menu').each(function () {
        $(this).remove();
      });

      var options = [];

      options.push({
        title: updateLeadLabel,
        id: 'lead_update',
        callback: function () {
          chrome.runtime.sendMessage({
            msg: 'integration_update_to_grabber', data: {
              integration: 'salesforce',
              specific: 'lead',
              lead_id: getObjectIdFromURL()
            }
          });
        }
      });

      createMenu(options);
      createMutationObserver();
    }, 100);
  }

  function createContactButtons() {
    if ($('.oneContent.active .slds-page-header .slds-button-group.oneActionsRibbon.forceActionsContainer').length === 0) {
      return;
    }

    setTimeout(function () {
      destroyMutationObserver();

      $('.snapaddy-button').each(function () {
        $(this).remove();
      });

      $('#snapaddy-dropdown-menu').each(function () {
        $(this).remove();
      });

      var org_id = getObjectIdFromURL();
      var options = [];

      options.push({
        title: updateContactLabel,
        id: 'contact_update',
        callback: function () {
          chrome.runtime.sendMessage({
            msg: 'integration_update_to_grabber', data: {
              integration: 'salesforce',
              specific: 'contact',
              contact_id: getObjectIdFromURL()
            }
          });
        }
      });

      if (org_id) {
        options.push({
          title: createContactLabel,
          id: 'contact_create_contact',
          callback: function () {
            chrome.runtime.sendMessage({
              msg: 'integration_update_to_grabber', data: {
                integration: 'salesforce',
                specific: 'contact_create_contact',
                contact_id: getObjectIdFromURL()
              }
            });
          }
        });
      }

      createMenu(options);
      createMutationObserver();
    }, 100);
  }

  function createAccountButtons() {
    if ($('.oneContent.active .slds-page-header .slds-button-group.oneActionsRibbon.forceActionsContainer').length === 0) {
      return;
    }

    setTimeout(function () {
      destroyMutationObserver();

      $('.snapaddy-button').each(function () {
        $(this).remove();
      });

      $('#snapaddy-dropdown-menu').each(function () {
        $(this).remove();
      });

      var org_id = getObjectIdFromURL();
      var options = [];

      options.push({
        title: createContactLabel,
        id: 'account_create_contact',
        callback: function () {
          chrome.runtime.sendMessage({
            msg: 'integration_update_to_grabber', data: {
              integration: 'salesforce',
              specific: 'account_create_contact',
              account_id: getObjectIdFromURL()
            }
          });
        }
      });

      if (org_id) {
        options.push({
          title: updateAccountLabel,
          id: 'account_update',
          callback: function () {
            chrome.runtime.sendMessage({
              msg: 'integration_update_to_grabber', data: {
                integration: 'salesforce',
                specific: 'account',
                account_id: getObjectIdFromURL()
              }
            });
          }
        });
      }

      createMenu(options);
      createMutationObserver();
    }, 100);
  }

  var createUpdateButton = function () {
    console.log('MutationObserved');
    chrome.storage.local.get('SNAPADDY_UPDATER_CONFIG', function (items) {
      let configArray = items.SNAPADDY_UPDATER_CONFIG;
      if (configArray && configArray.length > 0) {
        configArray.forEach((config) => {
          if (config.key === 'salesforce') {
            if (config.showUpdaterButton) {
              if (!config.disabledEntities.account && isActive('account')) {
                createAccountButtons();
              }

              if (!config.disabledEntities.contact && isActive('contact')) {
                createContactButtons();
              }

              if (!config.disabledEntities.lead && isActive('lead')) {
                createLeadButtons();
              }
            }
          }
        });
      }
    });
  };

  function createMutationObserver() {
    console.warn('Creating MutationObserver');
    mutationObserver = new MutationObserver(createUpdateButton);
    mutationObserver.observe(document.body, {
      subtree: true,
      childList: true
    });
  }

  function destroyMutationObserver() {
    mutationObserver.disconnect();
    mutationObserver = null;
  }

  createMutationObserver();

})();
